import os
import sys
import glob
import io
import difflib
import xml.etree.ElementTree as ET

version = 2.0

class TSRecord:
    name: str
    comment: str
    source: str
    translation: str
    def __init__(self, _name):
        self.name = _name
        self.comment = ""
        self.source = ""
        self.translation = ""

class Folders:
    firstDir = "first"
    secondDir = "second"
    result = "result"

def printVersion():
    print("TSFileComparing, version", version)
def createAllFolders(folder):
    if not os.path.exists(os.path.join(folder, Folders.result)):
        os.makedirs(os.path.join(folder, Folders.result))
    clearResultFolder(folder)


def clearResultFolder(folder):
    # clear the result folder
    files = glob.glob(os.path.join(folder, Folders.result, "*"))
    for f in files:
        os.remove(f)

def removeZeroResultFiles(folder):
    files = glob.glob(os.path.join(folder, Folders.result, "*"))
    for f in files:
        if os.path.getsize(f) == 0:
            os.remove(f)


def getDiff(folder, file1, file2):
    if os.path.exists(file1) and os.path.exists(file2):
        with open(file1, "r", encoding='utf8') as f1, open(file2, "r", encoding='utf8') as f2:
            f_1 = [i.strip() for i in f1]
            f_2 = [i.strip() for i in f2]

        res = difflib.context_diff(f_1, f_2)

        with open(os.path.join(folder, Folders.result, str(os.path.basename(file1) + "_diff")),
                  "w", encoding="utf-8") as output:
            output.writelines(res)


def generateDiffByFileNames(folder):
    imageFiles = []
    src = os.path.join(folder, Folders.firstDir)
    for file in glob.glob(os.path.join(src, "*.ts")):
        imageFiles.append(os.path.basename(file))
    for file in glob.glob(os.path.join(src, "*.xml")):
        imageFiles.append(os.path.basename(file))

    imageFiles2 = []
    src = os.path.join(folder, Folders.secondDir)
    for file in glob.glob(os.path.join(src, "*.ts")):
        imageFiles2.append(os.path.basename(file))
    for file in glob.glob(os.path.join(src, "*.xml")):
        imageFiles2.append(os.path.basename(file))

    files1 = []
    for fileName in imageFiles:
        file = os.path.join(folder, Folders.secondDir, fileName)
        if not os.path.exists(file):
            files1.append(fileName)

    files2 = []
    for fileName in imageFiles2:
        file = os.path.join(folder, Folders.firstDir, fileName)
        if not os.path.exists(file):
            files2.append(fileName)

    with open(os.path.join(folder, "add_removed_files.txt"), "w") as txt_file:
        if len(files1) or len(files2):
            txt_file.write("exists in first but don't exist in second:\n")
            for line in files1:
                txt_file.write(line + "\n")
            txt_file.write("exists in second but don't exist in first:\n")
            for line in files2:
                txt_file.write(line + "\n")
        else:
            txt_file.write("100% same\n")

def generateDiff(folder):
    files = []
    src = os.path.join(folder, Folders.firstDir)
    for file in glob.glob(os.path.join(src, "*.ts")):
        files.append(os.path.basename(file))
    for file in glob.glob(os.path.join(src, "*.xml")):
        files.append(os.path.basename(file))

    print("found files:", files)

    for fileName in files:
        getDiff(folder, os.path.join(folder, Folders.firstDir, fileName),
                os.path.join(folder, Folders.secondDir, fileName))

    removeZeroResultFiles(folder)

def generateDiffTS(folder):
    files = []
    src = os.path.join(folder, Folders.firstDir)
    for file in glob.glob(os.path.join(src, "*.ts")):
        files.append(os.path.basename(file))


    print("found ts files:", files)

    for fileName in files:
        getDiffTS(folder, os.path.join(folder, Folders.firstDir, fileName),
                os.path.join(folder, Folders.secondDir, fileName))

    removeZeroResultFiles(folder)

def readTSFromFile(file, tsrecords):
    root = ET.parse(file).getroot()

    for elem in root.findall('.//context'):
        # print(tag)
        a = ET.SubElement(elem, 'name')
        b = ET.SubElement(elem, 'location')
        c = ET.SubElement(b, 'filename')

        # print("!!!!!!")

        fullname = ""
        for name in elem.findall('.//name'):
            # print("name", name.text)
            if name.text:
                fullname = fullname + name.text

        for mes in elem.findall('.//message'):
            rec = TSRecord(fullname)

            for com in mes.findall('.//comment'):
                # print("  comment", com.text)
                rec.comment = com.text

            for loc in mes.findall('.//source'):
                # print("  source", loc.text)
                rec.source = loc.text

            for ts in mes.findall('.//translation'):
                # print("  translation", ts.text)
                rec.translation = ts.text

            tsrecords.append(rec)

        # print("---!!!!!!")

    return tsrecords

def getDiffTS(folder, file1, file2):
    if os.path.exists(file1) and os.path.exists(file2):
        recs1 = []
        recs2 = []
        recs1 = readTSFromFile(file1, recs1)
        recs2 = readTSFromFile(file2, recs2)

        resultLines = []
        resultLines.append("exist in the first but not in the second file:\n")
        for item in recs1:
            f = False
            for item2 in recs2:
                if item.comment == item2.comment:
                    f = True
                    break
            if not f:
                # print(item.comment)
                resultLines.append(item.comment + " " + item.name + " "
                                   + (item.source or "") + " " + (item.translation or "") + "\n")

        resultLines.append("exist in the second but not in the first file:\n")
        for item in recs2:
            f = False
            for item2 in recs1:
                if item.comment == item2.comment:
                    f = True
                    break
            if not f:
                # print(item.comment)
                resultLines.append(item.comment + " " + item.name + " "
                                   + (item.source or "") + " " + (item.translation or "") + "\n")

        if len(resultLines) > 2:
            with open(os.path.join(folder, Folders.result, str(os.path.basename(file1) + "_diff_nodes")),
                      "w", encoding="utf-8") as output:
                output.writelines(resultLines)


if __name__ == '__main__':
    printVersion()

    print("Run the script with 3 args - base folder, first folder name, second folder name. Example:")
    print("main.py d:\\Work\\Python\\tsfiles release develop")
    print("current args:", sys.argv)

    path = "d:\\Work\\Python\\tsfiles"
    Folders.firstDir = "first"
    Folders.secondDir = "second"
    # Folders.firstDir = "6.1"
    # Folders.secondDir = "6.2"

    if len(sys.argv) > 1:
        path = sys.argv[1]
        Folders.firstDir = sys.argv[2]
        Folders.secondDir = sys.argv[3]

    print("clear folders")
    createAllFolders(path)
    print("generate diffs")
    generateDiffByFileNames(path)

    generateDiff(path)
    generateDiffTS(path)

    exit(0)

