import os
import sys
import glob
import io
import re
import xml.etree.ElementTree as ET
import json

version = 1.1

class Folders:
    baseDir = "base"
    tsFile = "en_US.ts"
    languageJsonDir = "languageJson"

class TSRecord:
    name: str
    comment: str
    source: str
    translation: str
    location: str
    def __init__(self, _name):
        self.name = _name
        self.comment = ""
        self.source = ""
        self.translation = ""
        self.location = ""


def printVersion():
    print("TSFileComparing, version", version)



def readTSFromFile(file, tsrecords):
    root = ET.parse(file).getroot()

    for elem in root.findall('.//context'):
        # print(tag)

        # print("!!!!!!")

        fullname = ""
        for name in elem.findall('.//name'):
            # print("name", name.text)
            if name.text:
                fullname = fullname + name.text

        for mes in elem.findall('.//message'):
            rec = TSRecord(fullname)
            
            for com in mes.findall('.//comment'):
                # print("  comment", com.text)
                rec.comment = com.text

            for loc in mes.findall('.//source'):
                # print("  source", loc.text)
                rec.source = loc.text

            for ts in mes.findall('.//translation'):
                # print("  translation", ts.text)
                rec.translation = ts.text
            
            for val in mes.findall('.//location'):
                valAttr = val.attrib
                if 'filename' in valAttr:
                    # print(" location filename is ", valAttr['filename'])
                    rec.location = valAttr['filename']

                    

            tsrecords.append(rec)

        # print("---!!!!!!")

    return tsrecords

def searchJsonForStringIds(folderName, stringList):
    stringsAndFiles = {}

    for string in stringList:
        for fname in os.listdir(folderName):
            fullPathFName = os.path.join(folderName, fname)
            if os.path.isfile(fullPathFName):
                if fullPathFName.endswith(".json"):
                    # print("looking at file " + fname)
                    f = open(fullPathFName, 'r')
                    fData = f.read()
                    if string in fData:
                        iUndScore = fname.find("_")
                        shortName = fname[:iUndScore]
                        if string not in stringsAndFiles:
                            stringsAndFiles[string] = shortName
                        else:
                            currentScreens = stringsAndFiles[string]
                            currentScreens = currentScreens + " " + shortName
                            stringsAndFiles[string] = currentScreens
    
    return stringsAndFiles


def parseJsonForStringIds(folderName):
    stringIdValuesInJson = {}
    
    for fname in os.listdir(folderName):
            fullPathFName = os.path.join(folderName, fname)
            if os.path.isfile(fullPathFName):
                if fullPathFName.endswith(".json"):
                    # print("looking at file " + fname)
                    with open(fullPathFName) as screenFile:
                        screenData = json.load(screenFile)
                        stringIdValues = []
                        for entry in screenData:
                            if isinstance(screenData[entry], str):
                                stringIdValues.append(entry + ": " + screenData[entry])
                            else:
                                stringIdValues.append(entry)
                        iUndScore = fname.find("_")
                        shortName = fname[:iUndScore]
                        stringIdValuesInJson[shortName] = stringIdValues
    
    return stringIdValuesInJson    

if __name__ == '__main__':
    printVersion()

    print("Run the script with 3 args - base folder, TS file name, JSON folder name. Example:")
    print("main.py d:\\Work\\Python\\tsfiles xx_YY.tx languageJson")
    print("current args:", sys.argv)

    path = "d:\\Work\\Python\\tsfiles"

    if len(sys.argv) > 3:
        Folders.baseDir = sys.argv[1]
        Folders.tsFile = sys.argv[2]
        Folders.languageJsonDir = sys.argv[3]
    else:
        exit(1)

    if os.path.isfile(os.path.join(Folders.baseDir, "search-list.csv")):
        os.remove(os.path.join(Folders.baseDir, "search-list.csv"))

    tsFile = os.path.join(Folders.baseDir, Folders.tsFile)

    tsRecords = []
    tsList = []
    tsRecords = readTSFromFile(tsFile, tsRecords)
    for rec in tsRecords:
        if isinstance(rec.comment, str) and len(rec.comment) > 0:
            tsList.append(rec.comment)

    stringsInFiles = searchJsonForStringIds(os.path.join(Folders.baseDir, Folders.languageJsonDir), tsList)
    
    resultFile = open(os.path.join(Folders.baseDir, "search-list.csv"), 'w')
    resultFile.write("File,Source String,StringId,ScreensFound\n")
    for rec in tsRecords:
        source = str(rec.source).replace("\"", "")
        source = source.replace(",", "")
        location = str(rec.location).replace("../", "")
        location = location.replace("SavannaInstrument/", "")
        if isinstance(rec.comment, str) and rec.comment in stringsInFiles:
            resultFile.write(location + ",\"" + source + "\"," + rec.comment + ",\"" + stringsInFiles[rec.comment]  + "\",")
        else:
            resultFile.write(location + ",\"" + source + "\"," + str(rec.comment) + ",,")
        
        if "src/Simulator/" in location or "src/TrainingSimulator" in location:
            resultFile.write("Simulator Only\n")
        else:
            resultFile.write("\n")
    resultFile.close()

    stringIdInFiles = parseJsonForStringIds(os.path.join(Folders.baseDir, Folders.languageJsonDir))
    resultStringFile = open(os.path.join(Folders.baseDir, "search-result.csv"), 'w')
    resultStringFile.write("Screen,StringId values\n")
    for data in sorted(stringIdInFiles):
        resultStringFile.write(data)
        for value in stringIdInFiles[data]:
            resultStringFile.write(", " + value + "\n")
    resultStringFile.close()

    exit(0)

